/*
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package org.processmining.xeslite.lite.factory;

import gnu.trove.map.hash.THashMap;

import java.io.Serializable;
import java.util.Map;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;

import com.google.common.collect.ForwardingMap;

/**
 * @author F. Mannhardt
 * 
 */
class XAttributeMapLiteImpl extends ForwardingMap<String, XAttribute> implements XAttributeMap, Serializable {

	private static final long serialVersionUID = 6928328637653669033L;

	private Map<String, XAttribute> attributeMap;

	public XAttributeMapLiteImpl() {
	}

	public XAttributeMapLiteImpl(int initialSize) {
		attributeMap = createInternalMap(initialSize);
	}

	protected Map<String, XAttribute> createInternalMap() {
		return new THashMap<>(3, 0.5f);
	}

	protected Map<String, XAttribute> createInternalMap(int initialSize) {
		return new THashMap<>(initialSize, 0.5f);
	}

	@Override
	protected Map<String, XAttribute> delegate() {
		if (attributeMap == null) {
			attributeMap = createInternalMap();
		}
		return attributeMap;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#clone()
	 */
	public Object clone() {
		XAttributeMapLiteImpl clone = new XAttributeMapLiteImpl();
		if (!isEmpty()) {
			clone.attributeMap = createInternalMap(size());
			for (String key : keySet()) {
				clone.attributeMap.put(key, (XAttribute) get(key).clone());
			}
			return clone;
		}
		return clone;
	}

}
